#ifdef GL_ES
#define LOWP lowp
precision mediump float;
#else
#define LOWP
#endif

#define PI 3.14

//inputs from vertex shader
varying vec2 v_texCoord;
varying vec4 v_fragmentColor;

uniform vec2 u_lightLocation;
uniform vec3 u_lightColor;
uniform float u_radius;
uniform vec2 u_resolution;
//uniform sampler2D u_texture;
uniform float u_shadowMap[512];

// ----------------------------------------------------------
//sample from the 1D distance map
float sample(vec2 coord, float r) {
	//return step(r, texture2D(u_texture, coord).r);
	return step(r, u_shadowMap[int(coord.x * 512.0)]);
	//return u_shadowMap[int(coord.x * 512.0)];
}

// ----------------------------------------------------------
void main() {
	// rectangular to polar
	vec2 fragCoord  = gl_FragCoord.xy * 0.5; // (multiplied by inverse pixel size)
	
	vec2 offset     = vec2(u_lightLocation.x-(u_resolution.x/2.0), u_lightLocation.y-(u_resolution.y/2.0));
	//vec2 inputCoord = (vec2(fragCoord.x-offset.x, u_resolution.y - (fragCoord.y-offset.y))) / u_resolution;
	vec2 inputCoord = (vec2(fragCoord.x-offset.x, fragCoord.y-offset.y)) / u_resolution; // (x = 0 thru 1, y = 0 thru 1)
	//inputCoord.y = 1.0 - inputCoord.y;
	vec2 norm       = inputCoord * 2.0 - 1.0;
	//vec2 norm       = (inputCoord - 0.5) * 2.0; // x = -.5 thru .5, y = -.5 thru .5
	//vec2 norm = vec2(inputCoord.x - 0.5, inputCoord.y - 0.5);
	float theta     = atan(norm.y, norm.x);
	float r         = length(norm);
	//float coord     = (theta + PI) / (2.0*PI);
	//float coord     = mod(theta, PI) / (2.0*PI);
	float coord = mod(theta * 0.5, PI) / (2.0 * PI);

	vec2 tc = vec2(coord, 0.0);
	float center = sample(tc, r);
	float sum = center;
/*
	//we multiply the blur amount by our distance from center
	//this leads to more blurriness as the shadow "fades away"
	float blur = (1./u_resolution.x) * smoothstep(1.0, 0.0, r);

	// now we use a simple gaussian blur
	float sum = 0.0;
	sum += sample(vec2(tc.x - 4.0*blur, tc.y), r) * 0.05;
	sum += sample(vec2(tc.x - 3.0*blur, tc.y), r) * 0.09;
	sum += sample(vec2(tc.x - 2.0*blur, tc.y), r) * 0.12;
	sum += sample(vec2(tc.x - 1.0*blur, tc.y), r) * 0.15;
	sum += center * 0.16;
	sum += sample(vec2(tc.x + 1.0*blur, tc.y), r) * 0.15;
	sum += sample(vec2(tc.x + 2.0*blur, tc.y), r) * 0.12;
	sum += sample(vec2(tc.x + 3.0*blur, tc.y), r) * 0.09;
	sum += sample(vec2(tc.x + 4.0*blur, tc.y), r) * 0.05;
*/

	float distance = length(u_lightLocation - fragCoord);
	float intensity = clamp((u_radius / distance) * sum, 0.0, 1.0);
	if (intensity > 0.0)
		gl_FragColor = vec4(u_lightColor, 1.0) * vec4(intensity, intensity, intensity, 0.05);
	else
		gl_FragColor = vec4(0.0, 0.0, 0.0, 0.5);

	// gl_FragCoord = (0 through 2048 left to right, 0 through 1536 bottom to top)
	//gl_FragColor = vec4(0.0, gl_FragCoord.y / u_resolution.y, 0.0, 0.9);
	//float f = gl_FragCoord.x / (u_resolution.x * 1.0);
	/*float f = mod(theta * 0.5, PI) / PI;
	if (f < 0.0)
		f = 1.0;
	if (f > 1.0)
		f = 0.0;
	gl_FragColor = vec4(f, 0.0, 0.0, 0.9);*/
}
